@@ -27,6 +27,7 @@ from ovn_bgp_agent.drivers.openstack.utils import driver_utils
 from ovn_bgp_agent.drivers.openstack.utils import ovn
 from ovn_bgp_agent.drivers.openstack.utils import ovs
 from ovn_bgp_agent.drivers.openstack.utils import wire as wire_utils
+from ovn_bgp_agent.drivers.openstack.watchers import base_watcher
 from ovn_bgp_agent.drivers.openstack.watchers import bgp_watcher as watcher
 from ovn_bgp_agent import exceptions as agent_exc
 from ovn_bgp_agent.utils import helpers
@@ -112,8 +113,8 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
                   watcher.FIPUnsetEvent(self),
                   watcher.OVNLBMemberCreateEvent(self),
                   watcher.OVNLBMemberDeleteEvent(self),
-                  watcher.ChassisCreateEvent(self),
-                  watcher.ChassisPrivateCreateEvent(self),
+                  base_watcher.ChassisCreateEvent(self),
+                  base_watcher.ChassisPrivateCreateEvent(self),
                   watcher.LocalnetCreateDeleteEvent(self)}
         if self._expose_tenant_networks:
             events.update({watcher.SubnetRouterAttachedEvent(self),
@@ -335,7 +336,7 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
             if wire_utils.wire_provider_port(
                     self.ovn_routing_tables_routes, self.ovs_flows, port_ips,
                     bridge_device, bridge_vlan, localnet,
-                    self.ovn_routing_tables, proxy_cidrs, lladdr=lladdr):
+                    self.ovn_routing_tables, proxy_cidrs, mac=lladdr):
                 # Expose the IP now that it is connected
                 bgp_utils.announce_ips(port_ips)
                 return True
@@ -413,7 +414,7 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
             return wire_utils.unwire_provider_port(
                 self.ovn_routing_tables_routes, port_ips, bridge_device,
                 bridge_vlan, self.ovn_routing_tables, proxy_cidrs,
-                lladdr=lladdr)
+                mac=lladdr)
         except Exception as e:
             LOG.exception("Unexpected exception while unwiring provider port: "
                           "%s", e)
