@@ -391,14 +391,36 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
 
     def get_network_vlan_tag_by_network_name(self, network_name):
         tags = []
-        cmd = self.db_find_rows('Logical_Switch_Port', ('type', '=',
-                                constants.OVN_LOCALNET_VIF_PORT_TYPE))
-        for row in cmd.execute(check_error=True):
-            if (row.tag and row.options and
-                    row.options.get('network_name') == network_name):
+        for row in self.get_localnet_ports_by_network_name(network_name):
+            if row.tag:
                 tags.append(row.tag[0])
         return tags
 
+    def get_localnet_ports_by_network_name(self, network_name):
+        conditions = (
+            ('options', '=', {'network_name': network_name}),
+            ('type', '=', constants.OVN_LOCALNET_VIF_PORT_TYPE),
+        )
+        cmd = self.db_find_rows('Logical_Switch_Port', *conditions)
+        for row in cmd.execute(check_error=True):
+            yield row
+
+    def get_bgpvpn_networks_for_ports(self, ports,
+                                      vpn_type=constants.OVN_EVPN_TYPE_L3):
+        cmd = self.db_find_rows(
+            'Logical_Switch',
+            ('external_ids', '=', {constants.OVN_EVPN_TYPE_EXT_ID_KEY:
+                                   vpn_type}))
+        networks = []
+        for row in cmd.execute(check_error=True):
+            if not row.ports:
+                continue
+
+            localnet_ports = [p for p in row.ports if p in ports]
+            if localnet_ports:
+                networks.append(row)
+        return networks
+
     def ls_has_virtual_ports(self, logical_switch):
         ls = self.lookup('Logical_Switch', logical_switch)
         for port in ls.ports:
@@ -415,7 +437,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         ports = []
         cmd = self.db_find_rows('Logical_Switch_Port', ('up', '=', True))
         for row in cmd.execute(check_error=True):
-            port_chassis, _ = driver_utils.get_port_chassis(row, chassis)
+            port_chassis = driver_utils.get_port_chassis(row, chassis)
             if port_chassis == chassis:
                 ports.append(row)
 
@@ -777,21 +799,21 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
             lb = self.get_ovn_lb(lb_name)
             if not lb:
                 continue
-            lb_dp, lr_dp = helpers.get_lb_datapath_groups(lb)
-            if not lb_dp:
-                lb_dp = lb.datapaths
+            lb_dps, lr_dps = helpers.get_lb_datapaths(lb)
+            if not lb_dps:
+                lb_dps = lb.datapaths
 
-            if not lr_dp:
+            if not lr_dps:
                 # assume all the members are connected through the same router
                 # so only one datapath needs to be checked
-                router_lrps = self.get_lrps_for_datapath(lb_dp[0])
+                router_lrps = self.get_lrps_for_datapath(lb_dps[0])
                 for lrp in router_lrps:
                     router_lrp_dp = self.get_port_datapath(lrp)
                     if router_lrp_dp == router_dp:
                         lb_ip = ip_info.split(" ")[0].split("/")[0]
                         lbs[lb.name] = lb_ip
                         break
-            elif lr_dp == router_dp:
+            elif router_dp in lr_dps:
                 lb_ip = ip_info.split(" ")[0].split("/")[0]
                 lbs[lb.name] = lb_ip
 
