@@ -16,7 +16,11 @@ from oslo_concurrency import lockutils
 from oslo_log import log as logging
 
 from ovn_bgp_agent import constants
+from ovn_bgp_agent.drivers.openstack.utils import common as common_utils
 from ovn_bgp_agent.drivers.openstack.utils import driver_utils
+from ovn_bgp_agent.drivers.openstack.utils import loadbalancer as lb_utils
+from ovn_bgp_agent.drivers.openstack.utils import port as port_utils
+from ovn_bgp_agent.drivers.openstack.utils import router as router_utils
 from ovn_bgp_agent.drivers.openstack.watchers import base_watcher
 
 
@@ -49,11 +53,12 @@ class LogicalSwitchPortProviderCreateEvent(base_watcher.LSPChassisEvent):
             return
         try:
             # single and dual-stack format
-            if not self._check_ip_associated(row.addresses[0]):
+            if not port_utils.has_ip_address_defined(row.addresses[0]):
                 return False
 
-            current_chassis, _ = self._get_chassis(row)
-            logical_switch = self._get_network(row)
+            current_chassis = self._get_chassis(row)
+            logical_switch = common_utils.get_from_external_ids(
+                row, constants.OVN_LS_NAME_EXT_ID_KEY)
 
             if logical_switch in self.agent.ovn_local_lrps:
                 # This is a tenant network, routed through lrp, handled by
@@ -78,7 +83,7 @@ class LogicalSwitchPortProviderCreateEvent(base_watcher.LSPChassisEvent):
     def _run(self, event, row, old):
         with _SYNC_STATE_LOCK.read_lock():
             ips = row.addresses[0].split(' ')[1:]
-            ips_info = self._get_ips_info(row)
+            ips_info = port_utils.make_lsp_dict(row)
             self.agent.expose_ip(ips, ips_info)
 
 
@@ -100,11 +105,12 @@ class LogicalSwitchPortProviderDeleteEvent(base_watcher.LSPChassisEvent):
             return
         try:
             # single and dual-stack format
-            if not self._check_ip_associated(row.addresses[0]):
+            if not port_utils.has_ip_address_defined(row.addresses[0]):
                 return False
 
             ips = row.addresses[0].split(' ')[1:]
-            logical_switch = self._get_network(row)
+            logical_switch = common_utils.get_from_external_ids(
+                row, constants.OVN_LS_NAME_EXT_ID_KEY)
 
             if logical_switch in self.agent.ovn_local_lrps:
                 # This is a tenant network, routed through lrp, handled by
@@ -119,12 +125,12 @@ class LogicalSwitchPortProviderDeleteEvent(base_watcher.LSPChassisEvent):
             if event == self.ROW_DELETE:
                 return True
 
-            current_chassis, _ = self._get_chassis(row)
+            current_chassis = self._get_chassis(row)
             # Delete the port from current chassis, if
             # 1. port went down (while only attached here)
             if (hasattr(old, 'up') and bool(old.up[0]) and   # port was up
                     not bool(row.up[0]) and                  # is now down
-                    not self._has_additional_binding(row)):  # and bound here
+                    not port_utils.has_additional_binding(row)):  # and bound
                 return True
 
             # 2. port no longer bound here
@@ -136,7 +142,7 @@ class LogicalSwitchPortProviderDeleteEvent(base_watcher.LSPChassisEvent):
     def _run(self, event, row, old):
         with _SYNC_STATE_LOCK.read_lock():
             ips = row.addresses[0].split(' ')[1:]
-            ips_info = self._get_ips_info(row)
+            ips_info = port_utils.make_lsp_dict(row)
             self.agent.withdraw_ip(ips, ips_info)
 
 
@@ -180,10 +186,10 @@ class LogicalSwitchPortFIPCreateEvent(base_watcher.LSPChassisEvent):
             return
         try:
             # single and dual-stack format
-            if not self._check_ip_associated(row.addresses[0]):
+            if not port_utils.has_ip_address_defined(row.addresses[0]):
                 return False
 
-            current_chassis, _ = self._get_chassis(row)
+            current_chassis = self._get_chassis(row)
             current_port_fip = row.external_ids.get(
                 constants.OVN_FIP_EXT_ID_KEY)
             if (current_chassis != self.agent.chassis or
@@ -208,8 +214,10 @@ class LogicalSwitchPortFIPCreateEvent(base_watcher.LSPChassisEvent):
                 return False
 
             # Check if the current port_fip has not been exposed yet
-            return not self.agent.is_ip_exposed(self._get_network(row),
-                                                current_port_fip)
+            return not self.agent.is_ip_exposed(
+                common_utils.get_from_external_ids(
+                    row, constants.OVN_LS_NAME_EXT_ID_KEY),
+                current_port_fip)
 
         except (IndexError, AttributeError):
             return False
@@ -258,16 +266,17 @@ class LogicalSwitchPortFIPDeleteEvent(base_watcher.LSPChassisEvent):
             return
         try:
             # single and dual-stack format
-            if not self._check_ip_associated(row.addresses[0]):
+            if not port_utils.has_ip_address_defined(row.addresses[0]):
                 return False
 
-            current_port_fip = self._get_port_fip(row)
-            old_port_fip = self._get_port_fip(old)
+            current_port_fip = port_utils.get_fip(row)
+            old_port_fip = port_utils.get_fip(old)
             if not current_port_fip and not old_port_fip:
                 # This port is not a floating ip update
                 return False
 
-            logical_switch = self._get_network(row)
+            logical_switch = common_utils.get_from_external_ids(
+                row, constants.OVN_LS_NAME_EXT_ID_KEY)
             is_exposed = self.agent.is_ip_exposed(logical_switch,
                                                   old_port_fip or
                                                   current_port_fip)
@@ -291,7 +300,7 @@ class LogicalSwitchPortFIPDeleteEvent(base_watcher.LSPChassisEvent):
                 return True
 
             # If we reach here, just check if host changed
-            current_chassis, _ = self._get_chassis(row)
+            current_chassis = self._get_chassis(row)
             return current_chassis != self.agent.chassis
 
         except (IndexError, AttributeError):
@@ -300,17 +309,54 @@ class LogicalSwitchPortFIPDeleteEvent(base_watcher.LSPChassisEvent):
     def _run(self, event, row, old):
         # First check to remove the fip provided in old (since this might
         # have been updated)
-        fip = self._get_port_fip(old)
+        fip = port_utils.get_fip(old)
         if not fip:
             # Remove the fip provided in the current row, probably a
             # disassociate of the fip (or a down or a move)
-            fip = self._get_port_fip(row)
+            fip = port_utils.get_fip(row)
         if not fip:
             return
         with _SYNC_STATE_LOCK.read_lock():
             self.agent.withdraw_fip(fip, row)
 
 
+class LogicalSwitchUpdateEvent(base_watcher.LogicalSwitchChassisEvent):
+    '''Event to trigger on logical switch vrf config updates'''
+    def __init__(self, bgp_agent):
+        events = (self.ROW_UPDATE, self.ROW_DELETE)
+        super(LogicalSwitchUpdateEvent, self).__init__(
+            bgp_agent, events)
+
+    def match_fn(self, event, row, old):
+        '''Match updates for vrf configuration
+
+        Will trigger whenever external_ids[neutron_bgpvpn:vni] and
+        external_ids[neutron_bgpvpn:type] have been set and either one has
+        been updated
+        '''
+
+        settings = driver_utils.get_port_vrf_settings(row)
+        if settings and event == self.ROW_DELETE:
+            # Always run sync method if we are deleting this network (and it
+            # had settings applied)
+            return True
+
+        old_settings = driver_utils.get_port_vrf_settings(old)
+        if old_settings is None:
+            # it was not provided in old, so do not process this update
+            return False
+
+        return settings != old_settings
+
+    def _run(self, event, row, old):
+        with _SYNC_STATE_LOCK.read_lock():
+            # NOTE(mnederlof): For now it makes sense to run the sync method
+            # as this is triggered with a configured interval anyway and it
+            # will add/remove the triggered logical switch.
+            # It might make sense in the future to optimize this behaviour.
+            self.agent.sync()
+
+
 class LocalnetCreateDeleteEvent(base_watcher.LSPChassisEvent):
     def __init__(self, bgp_agent):
         events = (self.ROW_CREATE, self.ROW_DELETE,)
@@ -356,7 +402,7 @@ class ChassisRedirectCreateEvent(base_watcher.LRPChassisEvent):
 
     def _run(self, event, row, old):
         with _SYNC_STATE_LOCK.read_lock():
-            ips_info = self._get_ips_info(row)
+            ips_info = port_utils.make_lrp_dict(row)
             ips = [net.split("/")[0] for net in row.networks]
             self.agent.expose_ip(ips, ips_info)
 
@@ -390,7 +436,7 @@ class ChassisRedirectDeleteEvent(base_watcher.LRPChassisEvent):
 
     def _run(self, event, row, old):
         with _SYNC_STATE_LOCK.read_lock():
-            ips_info = self._get_ips_info(row)
+            ips_info = port_utils.make_lrp_dict(row)
             ips = [net.split("/")[0] for net in row.networks]
             self.agent.withdraw_ip(ips, ips_info)
 
@@ -441,7 +487,8 @@ class LogicalSwitchPortSubnetAttachEvent(base_watcher.LSPChassisEvent):
             subnet_info = {
                 'associated_router': row.external_ids.get(
                     constants.OVN_DEVICE_ID_EXT_ID_KEY),
-                'network': self._get_network(row),
+                'network': common_utils.get_from_external_ids(
+                    row, constants.OVN_LS_NAME_EXT_ID_KEY),
                 'address_scopes': driver_utils.get_addr_scopes(row)}
             self.agent.expose_subnet(ips, subnet_info)
 
@@ -516,7 +563,8 @@ class LogicalSwitchPortSubnetDetachEvent(base_watcher.LSPChassisEvent):
                 subnet_info = {
                     'associated_router': row.external_ids.get(
                         constants.OVN_DEVICE_ID_EXT_ID_KEY),
-                    'network': self._get_network(row),
+                    'network': common_utils.get_from_external_ids(
+                        row, constants.OVN_LS_NAME_EXT_ID_KEY),
                     'address_scopes': driver_utils.get_addr_scopes(row)}
             else:
                 associated_router = row.external_ids.get(
@@ -528,7 +576,8 @@ class LogicalSwitchPortSubnetDetachEvent(base_watcher.LSPChassisEvent):
                         associated_router = previous_associated_router
                 subnet_info = {
                     'associated_router': associated_router,
-                    'network': self._get_network(row),
+                    'network': common_utils.get_from_external_ids(
+                        row, constants.OVN_LS_NAME_EXT_ID_KEY),
                     'address_scopes': driver_utils.get_addr_scopes(row)}
             self.agent.withdraw_subnet(ips, subnet_info)
 
@@ -542,13 +591,14 @@ class LogicalSwitchPortTenantCreateEvent(base_watcher.LSPChassisEvent):
     def match_fn(self, event, row, old):
         try:
             # single and dual-stack format
-            if not self._check_ip_associated(row.addresses[0]):
+            if not port_utils.has_ip_address_defined(row.addresses[0]):
                 return False
 
             if not bool(row.up[0]):
                 return False
 
-            current_network = self._get_network(row)
+            current_network = common_utils.get_from_external_ids(
+                row, constants.OVN_LS_NAME_EXT_ID_KEY)
             if current_network not in self.agent.ovn_local_lrps:
                 return False
 
@@ -557,7 +607,8 @@ class LogicalSwitchPortTenantCreateEvent(base_watcher.LSPChassisEvent):
                     return True
 
             if hasattr(old, 'external_ids'):
-                old_network = self._get_network(old)
+                old_network = common_utils.get_from_external_ids(
+                    old, constants.OVN_LS_NAME_EXT_ID_KEY)
                 if old_network != current_network:
                     return True
         except (IndexError, AttributeError):
@@ -576,7 +627,8 @@ class LogicalSwitchPortTenantCreateEvent(base_watcher.LSPChassisEvent):
                 'cidrs': row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY,
                                               "").split(),
                 'type': row.type,
-                'logical_switch': self._get_network(row)
+                'logical_switch': common_utils.get_from_external_ids(
+                    row, constants.OVN_LS_NAME_EXT_ID_KEY),
             }
             self.agent.expose_remote_ip(ips, ips_info)
 
@@ -590,10 +642,11 @@ class LogicalSwitchPortTenantDeleteEvent(base_watcher.LSPChassisEvent):
     def match_fn(self, event, row, old):
         try:
             # single and dual-stack format
-            if not self._check_ip_associated(row.addresses[0]):
+            if not port_utils.has_ip_address_defined(row.addresses[0]):
                 return False
 
-            current_network = self._get_network(row)
+            current_network = common_utils.get_from_external_ids(
+                row, constants.OVN_LS_NAME_EXT_ID_KEY)
             # Assuming the current_network cannot be changed at once
             if current_network not in self.agent.ovn_local_lrps:
                 return False
@@ -620,7 +673,8 @@ class LogicalSwitchPortTenantDeleteEvent(base_watcher.LSPChassisEvent):
                 'cidrs': row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY,
                                               "").split(),
                 'type': row.type,
-                'logical_switch': self._get_network(row)
+                'logical_switch': common_utils.get_from_external_ids(
+                    row, constants.OVN_LS_NAME_EXT_ID_KEY),
             }
             self.agent.withdraw_remote_ip(ips, ips_info)
 
@@ -638,7 +692,7 @@ class OVNLBCreateEvent(base_watcher.OVNLBEvent):
         try:
             if not row.vips:
                 return False
-            lb_router = self._get_router(row)
+            lb_router = router_utils.get_name_from_external_ids(row)
             if lb_router not in self.agent.ovn_local_cr_lrps.keys():
                 return False
 
@@ -654,7 +708,7 @@ class OVNLBCreateEvent(base_watcher.OVNLBEvent):
 
             if hasattr(old, 'external_ids'):
                 # Check if the lb_router was added
-                old_lb_router = self._get_router(old)
+                old_lb_router = router_utils.get_name_from_external_ids(old)
                 if lb_router != old_lb_router:
                     return True
         except AttributeError:
@@ -663,12 +717,12 @@ class OVNLBCreateEvent(base_watcher.OVNLBEvent):
 
     def _run(self, event, row, old):
         # vips field grows
-        diff = self._get_diff_ip_from_vips(row, old)
+        diff = lb_utils.get_diff_ip_from_vips(row, old)
         for ip in diff:
             with _SYNC_STATE_LOCK.read_lock():
-                if self._is_vip(row, ip):
+                if lb_utils.is_vip(row, ip):
                     self.agent.expose_ovn_lb_vip(row)
-                elif self._is_fip(row, ip):
+                elif lb_utils.is_fip(row, ip):
                     self.agent.expose_ovn_lb_fip(row)
 
         # router set ext-gw
@@ -677,7 +731,8 @@ class OVNLBCreateEvent(base_watcher.OVNLBEvent):
         # since the subnet did not have access to the public network
         if hasattr(old, 'external_ids'):
             with _SYNC_STATE_LOCK.read_lock():
-                if self._get_router(old) != self._get_router(row):
+                if (router_utils.get_name_from_external_ids(old) !=
+                        router_utils.get_name_from_external_ids(row)):
                     self.agent.expose_ovn_lb_vip(row)
 
 
@@ -695,15 +750,15 @@ class OVNLBDeleteEvent(base_watcher.OVNLBEvent):
             if event == self.ROW_DELETE:
                 if not row.vips:
                     return False
-                lb_router = self._get_router(row)
+                lb_router = router_utils.get_name_from_external_ids(row)
                 if lb_router in self.agent.ovn_local_cr_lrps.keys():
                     return True
                 return False
 
             # ROW UPDATE EVENT
-            lb_router = self._get_router(row)
+            lb_router = router_utils.get_name_from_external_ids(row)
             if hasattr(old, 'external_ids'):
-                old_lb_router = self._get_router(old)
+                old_lb_router = router_utils.get_name_from_external_ids(old)
                 if not old_lb_router:
                     return False
                 if old_lb_router not in self.agent.ovn_local_cr_lrps.keys():
@@ -726,29 +781,30 @@ class OVNLBDeleteEvent(base_watcher.OVNLBEvent):
     def _run(self, event, row, old):
         # DELETE event need drop all
         if event == self.ROW_DELETE:
-            diff = self._get_ip_from_vips(row)
+            diff = lb_utils.get_vips(row)
             for ip in diff:
                 with _SYNC_STATE_LOCK.read_lock():
-                    if self._is_vip(row, ip):
+                    if lb_utils.is_vip(row, ip):
                         self.agent.withdraw_ovn_lb_vip(row)
-                    elif self._is_fip(row, ip):
+                    elif lb_utils.is_fip(row, ip):
                         self.agent.withdraw_ovn_lb_fip(row)
             return
 
         # UPDATE event
         # vips field decrease
-        diff = self._get_diff_ip_from_vips(old, row)
+        diff = lb_utils.get_diff_ip_from_vips(old, row)
         for ip in diff:
             with _SYNC_STATE_LOCK.read_lock():
-                if self._is_vip(old, ip):
+                if lb_utils.is_vip(old, ip):
                     self.agent.withdraw_ovn_lb_vip(old)
-                elif self._is_fip(old, ip):
+                elif lb_utils.is_fip(old, ip):
                     self.agent.withdraw_ovn_lb_fip(old)
 
         # router unset ext-gw
         if hasattr(old, 'external_ids'):
             with _SYNC_STATE_LOCK.read_lock():
-                if self._get_router(old) != self._get_router(row):
+                if (router_utils.get_name_from_external_ids(old) !=
+                        router_utils.get_name_from_external_ids(row)):
                     self.agent.withdraw_ovn_lb_vip(old)
 
 
@@ -770,7 +826,8 @@ class OVNPFBaseEvent(base_watcher.OVNLBEvent):
 
         if not row.vips:
             return False
-        lb_router = self._get_router(row, constants.OVN_LR_NAME_EXT_ID_KEY)
+        lb_router = router_utils.get_name_from_external_ids(
+            row, constants.OVN_LR_NAME_EXT_ID_KEY)
         return lb_router in self.agent.ovn_local_cr_lrps.keys()
 
 
